﻿
using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

namespace Salient.IO
{
    /// <copyright file="ResourceLocator.cs" author="Sky Sanders">
    ///
    /// Public Domain "License"
    /// 
    /// I, Sky Sanders, single-handedly wrote the code in this file, and as the legal author 
    /// (it was not a "work for hire") have placed it in the public domain.
    /// 
    /// That means I have disclaimed copyright of it. 
    /// 
    /// It also means that you may take the source code and use it for any purpose, commercial 
    /// or non-commercial, copyrighted, open-source, Free, free, or public domain, with no legal 
    /// obligation to acknowledge the borrowing/copying in any way. 
    /// 
    /// That said; attribution and/or feedback is always appreciated.
    ///
    /// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
    /// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
    /// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
    /// PARTICULAR PURPOSE.
    ///
    /// </copyright>
    /// <version>1.0</version>
    /// <author>Sky Sanders</author>
    /// <web>http://spikes.codeplex.com</web>
    /// <email>sky.sanders@gmail.com</email>
    /// <date>2010-01-19</date>
    /// 
    /// <summary>
    /// This static class resolves resource Uri and returns contents as byte or text.
    /// The resource uri can be any resolvable file path, relative, absolute or UNC or
    /// a resource protocol uri, e.g. res://myassembly/myresource to load resource from
    /// a specific assembly or res://*/myresource to search all assemblies in the current 
    /// AppDomain.
    /// 
    /// See Salient.IO.ResourceLocator.Tests for complete usage examples.
    /// 
    /// A FileNotFound exception is thrown if any problems arise. Check message and/or inner
    /// exception.
    /// 
    /// </summary>
    public static class ResourceLocator
    {

        /// <summary>
        /// Resolves file path or resource uri and returns byte array.
        /// </summary>
        /// <param name="uri">valid file path or resource Uri res://[*|assemblyname]/resourcename</param>
        /// <returns></returns>
        public static byte[] GetResource(string uri)
        {
            if (_cache.ContainsKey(uri))
            {
                return (byte[])_cache[uri];
            }

            byte[] returnValue = uri.StartsWith("res://", StringComparison.OrdinalIgnoreCase)
                                     ?
                                         GetAssemblyResource(uri)
                                     :
                                         GetFileResource(uri);

            _cache[uri] = returnValue;

            return returnValue;
        }
        /// <summary>
        /// Resolves file path or resource uri and returns text using default encoding.
        /// </summary>
        /// <param name="uri">valid file path or resource Uri res://[*|assemblyname]/resourcename</param>
        /// <returns></returns>
        public static string GetTextResource(string uri)
        {
            return GetTextResource(uri, Encoding.Default);
        }

        /// <summary>
        /// Resolves file path or resource uri and returns text with specified encoding.
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="uri">valid file path or resource Uri res://[*|assemblyname]/resourcename</param>
        /// <param name="encoding"></param>
        /// <returns></returns>
        /// <exception cref="FileNotFoundException"></exception>
        public static string GetTextResource(string uri, Encoding encoding)
        {
            byte[] bytes = GetResource(uri);
            string returnValue;
            using (MemoryStream mem = new MemoryStream(bytes))
            {
                using (StreamReader s = new StreamReader(mem, encoding))
                {
                    returnValue = s.ReadToEnd();
                }
            }
            return returnValue;
        }

        #region Private Implementation

        private const string STR_CouldNotLoadReferencedAssembly = "Could not load referenced assembly:{0}";
        private const string STR_InvalidResourceUri = "Invalid resource uri:{0}";
        private const string STR_ReferencedAssemblyResourceNotFound = "Referenced assembly resource not found:{0}";

        private const string UriRxPattern = @"^(?<scheme>(?<nscheme>[^:/\?#]+):)?(?<auth>"
                                            + @"//(?<nauth>[^/\?#]*))?(?<path>[^\?#]*)"
                                            + @"(?<query>\?(?<nquery>[^#]*))?"
                                            + @"(?<frag>#(?<nfrag>.*))?";

        /// <summary>
        /// Cache resolved resources for quick retrieval
        /// </summary>
        private static readonly Hashtable _cache = new Hashtable();

        private static readonly Regex UriRx = new Regex(UriRxPattern, RegexOptions.ExplicitCapture | RegexOptions.Compiled);



        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        private static byte[] GetAssemblyResource(string uri)
        {
            string assemblyName, resourcePath;

            bool isAssemblyResourceUri = TryParseAssemblyResourceUri(uri, out assemblyName, out resourcePath);

            if (!isAssemblyResourceUri || string.IsNullOrEmpty(assemblyName) || string.IsNullOrEmpty(resourcePath))
            {
                throw new FileNotFoundException(String.Format(STR_InvalidResourceUri, uri));
            }

            byte[] returnValue = null;

            if (assemblyName == "*")
            {
                Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
                foreach (Assembly assembly in assemblies)
                {
                    returnValue = GetResouceBytes(assembly, resourcePath);
                    if (returnValue != null)
                    {
                        break;
                    }
                }
            }
            else
            {
                try
                {
                    Assembly a = Assembly.Load(assemblyName);
                    returnValue = GetResouceBytes(a, resourcePath);
                }
                catch (Exception ex)
                {
                    throw new FileNotFoundException(String.Format(STR_CouldNotLoadReferencedAssembly, assemblyName), ex);
                }
            }

            if (returnValue == null)
            {
                throw new FileNotFoundException(String.Format(STR_ReferencedAssemblyResourceNotFound, uri));
            }

            return returnValue;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="assembly"></param>
        /// <param name="path"></param>
        /// <returns></returns>
        private static byte[] GetResouceBytes(Assembly assembly, string path)
        {
            byte[] returnValue = null;
            using (Stream s = assembly.GetManifestResourceStream(path))
            {
                if (s != null)
                {
                    returnValue = new byte[s.Length];
                    s.Read(returnValue, 0, (int)s.Length);
                }
            }

            return returnValue;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uri"></param>
        /// <returns></returns>
        private static byte[] GetFileResource(string uri)
        {
            if (!File.Exists(uri))
            {
                throw new FileNotFoundException(uri);
            }
            byte[] returnValue;
            using (FileStream s = File.Open(uri, FileMode.Open))
            {
                returnValue = new byte[s.Length];
                s.Read(returnValue, 0, (int)s.Length);
            }
            return returnValue;
        }

        /// <summary>
        /// Parses a resource Uri (res://[*|assembly]/resource path)
        /// Necessary because Uri class chokes on the wildcard.
        /// </summary>
        /// <param name="uri"></param>
        /// <param name="assemblyName"></param>
        /// <param name="resourcePath"></param>
        /// <returns></returns>
        private static bool TryParseAssemblyResourceUri(string uri, out string assemblyName, out string resourcePath)
        {
            assemblyName = null;
            resourcePath = null;

            Match m = UriRx.Match(uri);

            if (m.Success)
            {
                assemblyName = m.Groups["nauth"].Value;
                resourcePath = m.Groups["path"].Value.TrimStart(new[] { '/' });
            }

            return m.Success &&
                   string.Compare(m.Groups["nscheme"].Value, "res", true) == 0 &&
                   !string.IsNullOrEmpty(assemblyName) &&
                   !string.IsNullOrEmpty(resourcePath);
        } 
        #endregion
    }
}