using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using NUnit.Framework;

namespace Salient.Data.Tests
{
    public class DataObj
    {
        public string Name { get; set; }
        public int Age { get; set; }
    }

    [TestFixture]
    public class EnumerableDataReaderFixture
    {

        #region CTor Tests

        private static IEnumerable<DataObj> DataSource
        {
            get
            {
                return new List<DataObj>
                           {
                               new DataObj {Name = "1", Age = 16},
                               new DataObj {Name = "2", Age = 26},
                               new DataObj {Name = "3", Age = 36},
                               new DataObj {Name = "4", Age = 46}
                           };
            }
        }

        [Test]
        public void TestIEnumerableCtor()
        {
            var r = new EnumerableDataReader(DataSource, typeof(DataObj));
            while (r.Read())
            {
                var values = new object[2];
                int count = r.GetValues(values);
                Assert.AreEqual(2, count);

                values = new object[1];
                count = r.GetValues(values);
                Assert.AreEqual(1, count);

                values = new object[3];
                count = r.GetValues(values);
                Assert.AreEqual(2, count);

                Assert.IsInstanceOf(typeof(string), r.GetValue(0));
                Assert.IsInstanceOf(typeof(int), r.GetValue(1));

                Console.WriteLine("Name: {0}, Age: {1}", r.GetValue(0), r.GetValue(1));
            }
        }

        [Test]
        public void TestIEnumerableOfAnonymousType()
        {
            // create generic list
            Func<Type, IList> toGenericList =
                type => (IList)Activator.CreateInstance(typeof(List<>).MakeGenericType(new[] { type }));

            // create generic list of anonymous type
            IList listOfAnonymousType = toGenericList(new { Name = "1", Age = 16 }.GetType());

            listOfAnonymousType.Add(new { Name = "1", Age = 16 });
            listOfAnonymousType.Add(new { Name = "2", Age = 26 });
            listOfAnonymousType.Add(new { Name = "3", Age = 36 });
            listOfAnonymousType.Add(new { Name = "4", Age = 46 });

            var r = new EnumerableDataReader(listOfAnonymousType);
            while (r.Read())
            {
                var values = new object[2];
                int count = r.GetValues(values);
                Assert.AreEqual(2, count);

                values = new object[1];
                count = r.GetValues(values);
                Assert.AreEqual(1, count);

                values = new object[3];
                count = r.GetValues(values);
                Assert.AreEqual(2, count);

                Assert.IsInstanceOf(typeof(string), r.GetValue(0));
                Assert.IsInstanceOf(typeof(int), r.GetValue(1));

                Console.WriteLine("Name: {0}, Age: {1}", r.GetValue(0), r.GetValue(1));
            }
        }

        [Test]
        public void TestIEnumerableOfTCtor()
        {
            var r = new EnumerableDataReader(DataSource);
            while (r.Read())
            {
                var values = new object[2];
                int count = r.GetValues(values);
                Assert.AreEqual(2, count);

                values = new object[1];
                count = r.GetValues(values);
                Assert.AreEqual(1, count);

                values = new object[3];
                count = r.GetValues(values);
                Assert.AreEqual(2, count);

                Assert.IsInstanceOf(typeof(string), r.GetValue(0));
                Assert.IsInstanceOf(typeof(int), r.GetValue(1));

                Console.WriteLine("Name: {0}, Age: {1}", r.GetValue(0), r.GetValue(1));
            }
        } 
        #endregion
        private static IEnumerable<DataObj2> DataSource2
        {
            get
            {
                var returnValue = new List<DataObj2>
                                  {
                                      new DataObj2
                                          {
                                              BooleanProp = false,
                                              StringProp = "stringProp",
                                              ByteProp = 1,
                                              CharProp = 'c',
                                              DateTimeProp = new DateTime(2007, 12, 18),
                                              DecimalProp = (decimal) 100.1,
                                              DoubleProp = 100.2,
                                              FloatProp = (float) 100.3,
                                              GuidProp = new Guid("{C14E0CFE-0FFC-484c-A695-DB80D6F4FFD4}"),
                                              Int16Prop = 5,
                                              Int32Prop = 6,
                                              Int64Prop = 7,
                                              NullProp = null
                                          }
                                  };


                return returnValue;
            }
        }

        [Test, ExpectedException(typeof (NotImplementedException))]
        public void Calling_Depth()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            int d = r.Depth;
        }

        [Test]
        public void Calling_FieldCount()
        {
            var r = new EnumerableDataReader(DataSource2);
            Assert.AreEqual(13, r.FieldCount);
        }

        [Test]
        public void Calling_GetBoolean()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            Assert.AreEqual(false, r.GetBoolean(0));
            Assert.IsInstanceOf(typeof (bool), r.GetBoolean(0));
        }

        [Test]
        public void Calling_GetByte()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            Assert.AreEqual(1, r.GetByte(2));
            Assert.IsInstanceOf(typeof (byte), r.GetByte(2));
        }

        [Test, ExpectedException(typeof (NotImplementedException))]
        public void Calling_GetBytes()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            int i = 0;
            long fieldOffset = 0;
            byte[] buffer = null;
            int bufferOffset = 0;
            int length = 0;
            r.GetBytes(i, fieldOffset, buffer, bufferOffset, length);
        }

        [Test]
        public void Calling_GetChar()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetChar(3);
            Assert.AreEqual('c', value);
            Assert.IsInstanceOf(typeof (char), value);
        }

        [Test, ExpectedException(typeof (NotImplementedException))]
        public void Calling_GetChars()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            int i = 0;
            long fieldOffset = 0;
            char[] buffer = null;
            int bufferOffset = 0;
            int length = 0;
            r.GetChars(i, fieldOffset, buffer, bufferOffset, length);
        }

        [Test, ExpectedException(typeof (NotImplementedException))]
        public void Calling_GetData()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            r.GetData(9);
        }

        [Test]
        public void Calling_GetDataTypeName()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            string value = r.GetDataTypeName(0);
            Assert.AreEqual("Boolean", value);
        }

        [Test]
        public void Calling_GetDateTime()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetDateTime(4);
            Assert.AreEqual(new DateTime(2007, 12, 18), value);
            Assert.IsInstanceOf(typeof (DateTime), value);
        }

        [Test]
        public void Calling_GetDecimal()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetDecimal(5);
            Assert.AreEqual((decimal) 100.1, value);
            Assert.IsInstanceOf(typeof (decimal), value);
        }

        [Test]
        public void Calling_GetDouble()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetDouble(6);
            Assert.AreEqual(100.2, value);
            Assert.IsInstanceOf(typeof (double), value);
        }

        [Test]
        public void Calling_GetFieldType()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            Type type = r.GetFieldType(0);
            Assert.AreEqual(typeof (bool), type);
        }

        [Test]
        public void Calling_GetFloat()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetFloat(7);
            Assert.AreEqual((float) 100.3, value);
            Assert.IsInstanceOf(typeof (float), value);
        }

        [Test]
        public void Calling_GetGuid()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetGuid(8);
            Assert.AreEqual(new Guid("{C14E0CFE-0FFC-484c-A695-DB80D6F4FFD4}"), value);
            Assert.IsInstanceOf(typeof (Guid), value);
        }

        [Test]
        public void Calling_GetInt16()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetInt16(9);
            Assert.AreEqual((Int16) 5, value);
            Assert.IsInstanceOf(typeof (Int16), value);
        }

        [Test]
        public void Calling_GetInt32()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetInt32(10);
            Assert.AreEqual(6, value);
            Assert.IsInstanceOf(typeof (Int32), value);
        }

        [Test]
        public void Calling_GetInt64()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetInt64(11);
            Assert.AreEqual((Int64) 7, value);
            Assert.IsInstanceOf(typeof (Int64), value);
        }

        [Test]
        public void Calling_GetName()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            string result = r.GetName(0);
            Assert.AreEqual("BooleanProp", result);
        }

        [Test]
        public void Calling_GetOrdinal()
        {
            var r = new EnumerableDataReader(DataSource2);

            Assert.AreEqual(0, r.GetOrdinal("BooleanProp"));
        }


        [Test]
        public void Calling_GetSchemaTable()
        {
            var r = new EnumerableDataReader(DataSource2);
            object result = r.GetSchemaTable();
            Assert.IsInstanceOf(typeof (DataTable), result);
            var dt = (DataTable) result;
            Assert.AreEqual(13, dt.Columns.Count);

            Assert.AreEqual(typeof (bool), dt.Columns[0].DataType);
            Assert.AreEqual(typeof (string), dt.Columns[1].DataType);
            Assert.AreEqual(typeof (Byte), dt.Columns[2].DataType);
            Assert.AreEqual(typeof (Char), dt.Columns[3].DataType);
            Assert.AreEqual(typeof (DateTime), dt.Columns[4].DataType);
            Assert.AreEqual(typeof (decimal), dt.Columns[5].DataType);
            Assert.AreEqual(typeof (double), dt.Columns[6].DataType);
            Assert.AreEqual(typeof (float), dt.Columns[7].DataType);
            Assert.AreEqual(typeof (Guid), dt.Columns[8].DataType);
            Assert.AreEqual(typeof (Int16), dt.Columns[9].DataType);
            Assert.AreEqual(typeof (Int32), dt.Columns[10].DataType);
            Assert.AreEqual(typeof (Int64), dt.Columns[11].DataType);
            Assert.AreEqual(typeof (string), dt.Columns[12].DataType);
            Assert.AreEqual("BooleanProp", dt.Columns[0].ColumnName);
            Assert.AreEqual("StringProp", dt.Columns[01].ColumnName);
            Assert.AreEqual("ByteProp", dt.Columns[02].ColumnName);
            Assert.AreEqual("CharProp", dt.Columns[03].ColumnName);
            Assert.AreEqual("DateTimeProp", dt.Columns[04].ColumnName);
            Assert.AreEqual("DecimalProp", dt.Columns[05].ColumnName);
            Assert.AreEqual("DoubleProp", dt.Columns[06].ColumnName);
            Assert.AreEqual("FloatProp", dt.Columns[07].ColumnName);
            Assert.AreEqual("GuidProp", dt.Columns[08].ColumnName);
            Assert.AreEqual("Int16Prop", dt.Columns[09].ColumnName);
            Assert.AreEqual("Int32Prop", dt.Columns[10].ColumnName);
            Assert.AreEqual("Int64Prop", dt.Columns[11].ColumnName);
            Assert.AreEqual("NullProp", dt.Columns[12].ColumnName);
        }

        [Test]
        public void Calling_GetString()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetString(1);
            Assert.AreEqual("stringProp", value);
            Assert.IsInstanceOf(typeof (string), value);
        }

        [Test]
        public void Calling_GetValue()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            object value = r.GetValue(1);
            Assert.AreEqual("stringProp", value);
            Assert.IsInstanceOf(typeof (string), value);
        }

        [Test]
        public void Calling_GetValues()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            var values = new object[13];
            int result = r.GetValues(values);
            Assert.AreEqual(13, result);

            values = new object[15];
            result = r.GetValues(values);
            Assert.AreEqual(13, result);

            values = new object[5];
            result = r.GetValues(values);
            Assert.AreEqual(5, result);
        }

        [Test]
        public void Calling_IsClosed()
        {
            var r = new EnumerableDataReader(DataSource2);
            Assert.IsFalse(r.IsClosed);
            r.Close();
            Assert.IsTrue(r.IsClosed);
        }

        [Test]
        public void Calling_IsDBNull()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.Read();
            Assert.IsTrue(r.IsDBNull(12));
        }

        [Test, ExpectedException(typeof (NotImplementedException))]
        public void Calling_NextResult()
        {
            var r = new EnumerableDataReader(DataSource2);
            r.NextResult();
        }

        [Test]
        public void Calling_RecordsAffected()
        {
            var r = new EnumerableDataReader(DataSource2);
            Assert.AreEqual(-1, r.RecordsAffected);
            r.Read();
        }

        [Test]
        public void Get_Name()
        {
            var r = new EnumerableDataReader(DataSource2);
            Assert.AreEqual("BooleanProp", r.GetName(0));
        }

        [Test]
        public void ReadTwice()
        {
            var r = new EnumerableDataReader(DataSource2);
            Assert.IsTrue(r.Read());
            Assert.IsFalse(r.Read(), "should be finished");
        }
    }
}